@@ -19,9 +19,7 @@
     Home: https://github.com/gorhill/uBlock
 */
 
-/* jshint esversion:11 */
-
-'use strict';
+import { ubolErr } from './debug.js';
 
 /******************************************************************************/
 
@@ -29,7 +27,7 @@ function fetchJSON(path) {
     return fetch(`${path}.json`).then(response =>
         response.json()
     ).catch(reason => {
-        console.info(reason);
+        ubolErr(`fetchJSON/${reason}`);
     });
 }
 
