@@ -20,6 +20,7 @@
 */
 
 import { dom, qs$, qsa$ } from './dom.js';
+import { hashFromIterable, nodeFromTemplate } from './dashboard.js';
 import { i18n, i18n$ } from './i18n.js';
 import { localRead, localWrite, sendMessage } from './ext.js';
 
@@ -28,7 +29,7 @@ import { localRead, localWrite, sendMessage } from './ext.js';
 export const rulesetMap = new Map();
 
 let cachedRulesetData = {};
-let hideUnusedSet = new Set([ 'regions' ]);
+let hideUnusedSet = new Set([ 'ads', 'regions' ]);
 
 /******************************************************************************/
 
@@ -90,14 +91,10 @@ function updateNodes(listEntries) {
 /******************************************************************************/
 
 function rulesetStats(rulesetId) {
-    const hasOmnipotence = cachedRulesetData.defaultFilteringMode > 1;
     const rulesetDetails = rulesetMap.get(rulesetId);
     if ( rulesetDetails === undefined ) { return; }
     const { rules, filters } = rulesetDetails;
-    let ruleCount = rules.plain + rules.regex;
-    if ( hasOmnipotence ) {
-        ruleCount += rules.removeparam + rules.redirect + rules.modifyHeaders;
-    }
+    const ruleCount = rules.plain + rules.regex;
     const filterCount = filters.accepted;
     return { ruleCount, filterCount };
 }
@@ -136,7 +133,7 @@ export function renderFilterLists(rulesetData) {
         if ( ruleset.homeURL ) {
             dom.attr(qs$(listEntry, 'a.support'), 'href', ruleset.homeURL);
         }
-        dom.cl.toggle(listEntry, 'isDefault', ruleset.id === 'default');
+        dom.cl.toggle(listEntry, 'isDefault', ruleset.enabled === true);
         const stats = rulesetStats(ruleset.id);
         if ( stats === undefined ) { return; }
         listEntry.title = listStatsTemplate
@@ -144,11 +141,10 @@ export function renderFilterLists(rulesetData) {
             .replace('{{filterCount}}', renderNumber(stats.filterCount));
         const fromAdmin = isAdminRuleset(ruleset.id);
         dom.cl.toggle(listEntry, 'fromAdmin', fromAdmin);
-        const disabled = stats.ruleCount === 0 || fromAdmin;
         dom.attr(
             qs$(listEntry, '.input.checkbox input'),
             'disabled',
-            disabled ? '' : null
+            fromAdmin ? '' : null
         );
     };
 
@@ -166,16 +162,16 @@ export function renderFilterLists(rulesetData) {
 
     const createListEntry = (listDetails, depth) => {
         if ( listDetails.lists === undefined ) {
-            return dom.clone('#templates .listEntry[data-role="leaf"]');
+            return nodeFromTemplate('listEntryLeaf', '.listEntry');
         }
         if ( depth !== 0 ) {
-            return dom.clone('#templates .listEntry[data-role="node"]');
+            return nodeFromTemplate('listEntryNode', '.listEntry');
         }
-        return dom.clone('#templates .listEntry[data-role="rootnode"]');
+        return nodeFromTemplate('listEntryRoot', '.listEntry');
     };
 
     const createListEntries = (parentkey, listTree, depth = 0) => {
-        const listEntries = dom.clone('#templates .listEntries');
+        const listEntries = nodeFromTemplate('listEntries', '.listEntries');
         const treeEntries = Object.entries(listTree);
         if ( depth !== 0 ) {
             const reEmojis = /\p{Emoji}+/gu;
@@ -216,9 +212,18 @@ export function renderFilterLists(rulesetData) {
         [
             'default',
             rulesetDetails.filter(ruleset =>
-                ruleset.id === 'default' ||
                 ruleset.group === 'default'
             ),
+        ], [
+            'ads',
+            rulesetDetails.filter(ruleset =>
+                ruleset.group === 'ads'
+            ),
+        ], [
+            'privacy',
+            rulesetDetails.filter(ruleset =>
+                ruleset.group === 'privacy'
+            ),
         ], [
             'malware',
             rulesetDetails.filter(ruleset =>
@@ -232,7 +237,6 @@ export function renderFilterLists(rulesetData) {
         ], [
             'misc',
             rulesetDetails.filter(ruleset =>
-                ruleset.id !== 'default' &&
                 ruleset.group === undefined &&
                 typeof ruleset.lang !== 'string' 
             ),
@@ -390,6 +394,8 @@ dom.on('#findInLists', 'input', searchFilterLists);
 
 const applyEnabledRulesets = (( ) => {
     const apply = async ( ) => {
+        dom.cl.add(dom.body, 'committing');
+
         const enabledRulesets = [];
         for ( const liEntry of qsa$('#lists .listEntry[data-role="leaf"][data-rulesetid]') ) {
             const checked = qs$(liEntry, 'input[type="checkbox"]:checked') !== null;
@@ -401,12 +407,17 @@ const applyEnabledRulesets = (( ) => {
 
         dom.cl.remove('#lists .listEntry.toggled', 'toggled');
 
-        if ( enabledRulesets.length === 0 ) { return; }
+        const modified = hashFromIterable(enabledRulesets) !==
+            hashFromIterable(cachedRulesetData.enabledRulesets);
+        if ( modified ) {
+            const result = await sendMessage({
+                what: 'applyRulesets',
+                enabledRulesets,
+            });
+            dom.text('#dnrError', result?.error || '');
+        }
 
-        await sendMessage({
-            what: 'applyRulesets',
-            enabledRulesets,
-        });
+        dom.cl.remove(dom.body, 'committing');
     };
 
     let timer;
@@ -424,7 +435,11 @@ const applyEnabledRulesets = (( ) => {
         }
         timer = self.setTimeout(( ) => {
             timer = undefined;
-            apply();
+            if ( dom.cl.has(dom.body, 'committing') ) {
+                applyEnabledRulesets();
+            } else {
+                apply();
+            }
         }, 997);
     }
 })();
