@@ -21,6 +21,7 @@
 
 import {
     broadcastMessage,
+    hasBroadHostPermissions,
     hostnamesFromMatches,
     isDescendantHostnameOfIter,
     toBroaderHostname,
@@ -32,6 +33,11 @@ import {
     sessionRead, sessionWrite,
 } from './ext.js';
 
+import {
+    rulesetConfig,
+    saveRulesetConfig,
+} from './config.js';
+
 import { adminReadEx } from './admin.js';
 import { filteringModesToDNR } from './ruleset-manager.js';
 
@@ -47,6 +53,13 @@ export const    MODE_BASIC = 1;
 export const  MODE_OPTIMAL = 2;
 export const MODE_COMPLETE = 3;
 
+export const defaultFilteringModes = {
+    none: [],
+    basic: [],
+    optimal: [ 'all-urls' ],
+    complete: [],
+};
+
 /******************************************************************************/
 
 const pruneDescendantHostnamesFromSet = (hostname, hnSet) => {
@@ -218,15 +231,29 @@ export async function readFilteringModeDetails(bypassCache = false) {
             return readFilteringModeDetails.cache;
         }
     }
-    let [ userModes, adminNoFiltering ] = await Promise.all([
+    let [
+        userModes = structuredClone(defaultFilteringModes),
+        adminDefaultFiltering,
+        adminNoFiltering,
+    ] = await Promise.all([
         localRead('filteringModeDetails'),
+        adminReadEx('defaultFiltering'),
         adminReadEx('noFiltering'),
     ]);
-    if ( userModes === undefined ) {
-        userModes = { basic: [ 'all-urls' ] };
-    }
     userModes = unserializeModeDetails(userModes);
-    if ( Array.isArray(adminNoFiltering) ) {
+    if ( adminDefaultFiltering !== undefined ) {
+        const modefromName = {
+            none: MODE_NONE,
+            basic: MODE_BASIC,
+            optimal: MODE_OPTIMAL,
+            complete: MODE_COMPLETE,
+        };
+        const adminDefaultFilteringMode = modefromName[adminDefaultFiltering];
+        if ( adminDefaultFilteringMode !== undefined ) {
+            applyFilteringMode(userModes, 'all-urls', adminDefaultFilteringMode);
+        }
+    }
+    if ( Array.isArray(adminNoFiltering) && adminNoFiltering.length !== 0 ) {
         if ( adminNoFiltering.includes('-*') ) {
             userModes.none.clear();
         }
@@ -254,27 +281,34 @@ async function writeFilteringModeDetails(afterDetails) {
     readFilteringModeDetails.cache = unserializeModeDetails(data);
     return Promise.all([
         getDefaultFilteringMode(),
-        getTrustedSites(),
+        hasBroadHostPermissions(),
         localWrite('filteringModeDetails', data),
         sessionWrite('filteringModeDetails', data),
     ]).then(results => {
         broadcastMessage({
             defaultFilteringMode: results[0],
-            trustedSites: Array.from(results[1]),
+            hasOmnipotence: results[1],
+            filteringModeDetails: readFilteringModeDetails.cache,
         });
     });
 }
 
 /******************************************************************************/
 
-export async function getFilteringModeDetails() {
+export async function getFilteringModeDetails(serializable = false) {
     const actualDetails = await readFilteringModeDetails();
-    return {
+    const out = {
         none: new Set(actualDetails.none),
         basic: new Set(actualDetails.basic),
         optimal: new Set(actualDetails.optimal),
         complete: new Set(actualDetails.complete),
     };
+    return serializable ? serializeModeDetails(out) : out;
+}
+
+export async function setFilteringModeDetails(details) {
+    await localWrite('filteringModeDetails', serializeModeDetails(details));
+    await readFilteringModeDetails(true);
 }
 
 /******************************************************************************/
@@ -303,66 +337,51 @@ export function setDefaultFilteringMode(afterLevel) {
 
 /******************************************************************************/
 
-export async function getTrustedSites() {
-    const filteringModes = await getFilteringModeDetails();
-    return filteringModes.none;
-}
-
-export async function setTrustedSites(hostnames) {
-    const filteringModes = await getFilteringModeDetails();
-    const { none } = filteringModes;
-    const hnSet = new Set(hostnames);
-    let modified = false;
-    // Set default mode to Basic when removing No-filtering as default mode
-    if ( none.has('all-urls') && hnSet.has('all-urls') === false ) {
-        applyFilteringMode(filteringModes, 'all-urls', MODE_BASIC);
-        modified = true;
-    }
-    for ( const hn of none ) {
-        if ( hnSet.has(hn) ) {
-            hnSet.delete(hn);
-        } else {
-            none.delete(hn);
-            modified = true;
-        }
-    }
-    for ( const hn of hnSet ) {
-        const level = applyFilteringMode(filteringModes, hn, MODE_NONE);
-        if ( level !== MODE_NONE ) { continue; }
-        modified = true;
-    }
-    if ( modified === false ) { return; }
-    return writeFilteringModeDetails(filteringModes);
-}
-
-/******************************************************************************/
-
 export async function syncWithBrowserPermissions() {
-    const [ permissions, beforeMode ] = await Promise.all([
+    const [
+        permissions,
+        beforeMode,
+    ] = await Promise.all([
         browser.permissions.getAll(),
         getDefaultFilteringMode(),
     ]);
     const allowedHostnames = new Set(hostnamesFromMatches(permissions.origins || []));
+    const hasBroadHostPermissions = allowedHostnames.has('all-urls');
+    const broadHostPermissionsToggled =
+        hasBroadHostPermissions !== rulesetConfig.hasBroadHostPermissions;
     let modified = false;
-    if ( beforeMode > MODE_BASIC && allowedHostnames.has('all-urls') === false ) {
+    if ( beforeMode > MODE_BASIC && hasBroadHostPermissions === false ) {
         await setDefaultFilteringMode(MODE_BASIC);
         modified = true;
+    } else if ( beforeMode === MODE_BASIC && hasBroadHostPermissions && broadHostPermissionsToggled ) {
+        await setDefaultFilteringMode(MODE_OPTIMAL);
+        modified = true;
+    }
+    if ( broadHostPermissionsToggled ) {
+        rulesetConfig.hasBroadHostPermissions = hasBroadHostPermissions;
+        saveRulesetConfig();
     }
     const afterMode = await getDefaultFilteringMode();
-    if ( afterMode > MODE_BASIC ) { return false; }
+    if ( afterMode > MODE_BASIC ) { return afterMode !== beforeMode; }
     const filteringModes = await getFilteringModeDetails();
-    const { optimal, complete } = filteringModes;
-    for ( const hn of optimal ) {
-        if ( allowedHostnames.has(hn) ) { continue; }
-        optimal.delete(hn);
-        modified = true;
+    if ( allowedHostnames.has('all-urls') === false ) {
+        const { optimal, complete } = filteringModes;
+        for ( const hn of optimal ) {
+            if ( allowedHostnames.has(hn) ) { continue; }
+            if ( isDescendantHostnameOfIter(hn, allowedHostnames) ) { continue; }
+            optimal.delete(hn);
+            modified = true;
+        }
+        for ( const hn of complete ) {
+            if ( allowedHostnames.has(hn) ) { continue; }
+            if ( isDescendantHostnameOfIter(hn, allowedHostnames) ) { continue; }
+            complete.delete(hn);
+            modified = true;
+        }
+        if ( modified ) {
+            await writeFilteringModeDetails(filteringModes);
+        }
     }
-    for ( const hn of complete ) {
-        if ( allowedHostnames.has(hn) ) { continue; }
-        complete.delete(hn);
-        modified = true;
-    }
-    await writeFilteringModeDetails(filteringModes);
     return modified;
 }
 
