@@ -20,38 +20,53 @@
 */
 
 import {
-    TAB_ID_NONE,
-    browser,
-    dnr,
     i18n,
     localRead, localRemove, localWrite,
     runtime,
     sessionRead, sessionRemove, sessionWrite,
+    webextFlavor,
 } from './ext.js';
 
 import {
     rulesetConfig,
     saveRulesetConfig,
 } from './config.js';
+import { ubolErr, ubolLog } from './debug.js';
 
-
+import { dnr } from './ext-compat.js';
 import { fetchJSON } from './fetch.js';
 import { getAdminRulesets } from './admin.js';
-import { ubolLog } from './debug.js';
+import { hasBroadHostPermissions } from './utils.js';
+import { rulesFromText } from './dnr-parser.js';
 
 /******************************************************************************/
 
+const SPECIAL_RULES_REALM = 5000000;
+const USER_RULES_BASE_RULE_ID = 9000000;
+const USER_RULES_PRIORITY = 1000000;
 const TRUSTED_DIRECTIVE_BASE_RULE_ID = 8000000;
+const TRUSTED_DIRECTIVE_PRIORITY = USER_RULES_PRIORITY + 1000000;
 const STRICTBLOCK_PRIORITY = 29;
 
+let dynamicRegexCount = 0;
+let sessionRegexCount = 0;
+
 /******************************************************************************/
 
 const isStrictBlockRule = rule => {
     if ( rule.priority !== STRICTBLOCK_PRIORITY ) { return false; }
-    if ( rule.action.type !== 'redirect' ) { return false; }
-    const substitution = rule.action.redirect.regexSubstitution;
-    return substitution !== undefined &&
-        substitution.includes('/strictblock.');
+    if ( rule.condition?.resourceTypes === undefined ) { return false; }
+    if ( rule.condition.resourceTypes.length !== 1 ) { return false; }
+    if ( rule.condition.resourceTypes[0] !== 'main_frame' ) { return false; }
+    if ( rule.action.type === 'redirect' ) {
+        const substitution = rule.action.redirect.regexSubstitution;
+        return substitution !== undefined &&
+            substitution.includes('/strictblock.');
+    }
+    if ( rule.action.type === 'allow' ) {
+        return Array.isArray(rule.condition?.requestDomains);
+    }
+    return false;
 };
 
 /******************************************************************************/
@@ -60,26 +75,22 @@ function getRulesetDetails() {
     if ( getRulesetDetails.rulesetDetailsPromise !== undefined ) {
         return getRulesetDetails.rulesetDetailsPromise;
     }
-    getRulesetDetails.rulesetDetailsPromise = fetchJSON('/rulesets/ruleset-details').then(entries => {
-        const rulesMap = new Map(
-            entries.map(entry => [ entry.id, entry ])
-        );
-        return rulesMap;
-    });
+    getRulesetDetails.rulesetDetailsPromise =
+        fetchJSON('/rulesets/ruleset-details').then(entries => {
+            const rulesMap = new Map(entries.map(entry => [ entry.id, entry ]));
+            return rulesMap;
+        });
     return getRulesetDetails.rulesetDetailsPromise;
 }
 
 /******************************************************************************/
 
-async function pruneInvalidRegexRules(realm, rulesIn) {
-    const rejectedRegexRules = [];
-
+async function pruneInvalidRegexRules(realm, rulesIn, rejected = []) {
     const validateRegex = regex => {
         return dnr.isRegexSupported({ regex, isCaseSensitive: false }).then(result => {
-            const isSupported = result?.isSupported || false;
-            pruneInvalidRegexRules.validated.set(regex, isSupported);
-            if ( isSupported ) { return true; }
-            rejectedRegexRules.push(`\t${regex}  ${result?.reason}`);
+            pruneInvalidRegexRules.validated.set(regex, result?.reason || true);
+            if ( result.isSupported ) { return true; }
+            rejected.push({ regex, reason: result?.reason });
             return false;
         });
     };
@@ -92,8 +103,11 @@ async function pruneInvalidRegexRules(realm, rulesIn) {
             continue;
         }
         const { regexFilter } = rule.condition;
-        if ( pruneInvalidRegexRules.validated.has(regexFilter) ) {
-            toCheck.push(pruneInvalidRegexRules.validated.get(regexFilter));
+        const reason = pruneInvalidRegexRules.validated.get(regexFilter);
+        if ( reason !== undefined ) {
+            toCheck.push(reason === true);
+            if ( reason === true  ) { continue; }
+            rejected.push({ regex: regexFilter, reason });
             continue;
         }
         toCheck.push(validateRegex(regexFilter));
@@ -102,9 +116,9 @@ async function pruneInvalidRegexRules(realm, rulesIn) {
     // Collate results
     const isValid = await Promise.all(toCheck);
 
-    if ( rejectedRegexRules.length !== 0 ) {
+    if ( rejected.length !== 0 ) {
         ubolLog(`${realm} realm: rejected regexes:\n`,
-            rejectedRegexRules.join('\n')
+            rejected.map(e => `${e.regex} → ${e.reason}`).join('\n')
         );
     }
 
@@ -117,8 +131,8 @@ pruneInvalidRegexRules.validated = new Map();
 async function updateRegexRules(currentRules, addRules, removeRuleIds) {
     // Remove existing regex-related block rules
     for ( const rule of currentRules ) {
-        const { type } = rule.action;
-        if ( type !== 'block' && type !== 'allow' ) { continue; }
+        if ( rule.id === 0 ) { continue; }
+        if ( rule.id >= SPECIAL_RULES_REALM ) { continue; }
         if ( rule.condition.regexFilter === undefined ) { continue; }
         removeRuleIds.push(rule.id);
     }
@@ -152,183 +166,64 @@ async function updateRegexRules(currentRules, addRules, removeRuleIds) {
 
 /******************************************************************************/
 
-async function updateRemoveparamRules(currentRules, addRules, removeRuleIds) {
-    // Remove existing removeparam-related rules
-    for ( const rule of currentRules ) {
-        if ( rule.action.type !== 'redirect' ) { continue; }
-        if ( rule.action.redirect.transform === undefined ) { continue; }
-        removeRuleIds.push(rule.id);
-    }
-
-    const [
-        hasOmnipotence,
-        rulesetDetails,
-    ] = await Promise.all([
-        browser.permissions.contains({ origins: [ '<all_urls>' ] }),
-        getEnabledRulesetsDetails(),
-    ]);
-
-    // Fetch removeparam rules for all enabled rulesets
-    const toFetch = [];
-    for ( const details of rulesetDetails ) {
-        if ( details.rules.removeparam === 0 ) { continue; }
-        toFetch.push(fetchJSON(`/rulesets/removeparam/${details.id}`));
-    }
-    const removeparamRulesets = await Promise.all(toFetch);
-
-    // Removeparam rules can only be enforced with omnipotence
-    const allRules = [];
-    if ( hasOmnipotence ) {
-        for ( const rules of removeparamRulesets ) {
-            if ( Array.isArray(rules) === false ) { continue; }
-            for ( const rule of rules ) {
-                allRules.push(rule);
-            }
-        }
-    }
-    if ( allRules.length === 0 ) { return; }
-
-    const validRules = await pruneInvalidRegexRules('removeparam', allRules);
-    if ( validRules.length === 0 ) { return; }
-
-    ubolLog(`Add ${validRules.length} DNR removeparam rules`);
-    addRules.push(...validRules);
-}
-
-/******************************************************************************/
-
-async function updateRedirectRules(currentRules, addRules, removeRuleIds) {
-    // Remove existing redirect-related rules
-    for ( const rule of currentRules ) {
-        if ( rule.action.type !== 'redirect' ) { continue; }
-        if ( rule.action.redirect.extensionPath === undefined ) { continue; }
-        removeRuleIds.push(rule.id);
-    }
-
-    const [
-        hasOmnipotence,
-        rulesetDetails,
-    ] = await Promise.all([
-        browser.permissions.contains({ origins: [ '<all_urls>' ] }),
-        getEnabledRulesetsDetails(),
-    ]);
-
-    // Fetch redirect rules for all enabled rulesets
-    const toFetch = [];
-    for ( const details of rulesetDetails ) {
-        if ( details.rules.redirect === 0 ) { continue; }
-        toFetch.push(fetchJSON(`/rulesets/redirect/${details.id}`));
-    }
-    const redirectRulesets = await Promise.all(toFetch);
-
-    // Redirect rules can only be enforced with omnipotence
-    const allRules = [];
-    if ( hasOmnipotence ) {
-        for ( const rules of redirectRulesets ) {
-            if ( Array.isArray(rules) === false ) { continue; }
-            for ( const rule of rules ) {
-                allRules.push(rule);
-            }
-        }
-    }
-    if ( allRules.length === 0 ) { return; }
-
-    const validRules = await pruneInvalidRegexRules('redirect', allRules);
-    if ( validRules.length === 0 ) { return; }
-
-    ubolLog(`Add ${validRules.length} DNR redirect rules`);
-    addRules.push(...validRules);
-}
-
-/******************************************************************************/
-
-async function updateModifyHeadersRules(currentRules, addRules, removeRuleIds) {
-    // Remove existing header modification-related rules
-    for ( const rule of currentRules ) {
-        if ( rule.action.type !== 'modifyHeaders' ) { continue; }
-        removeRuleIds.push(rule.id);
-    }
-
-    const [
-        hasOmnipotence,
-        rulesetDetails,
-    ] = await Promise.all([
-        browser.permissions.contains({ origins: [ '<all_urls>' ] }),
-        getEnabledRulesetsDetails(),
-    ]);
-
-    // Fetch modifyHeaders rules for all enabled rulesets
-    const toFetch = [];
-    for ( const details of rulesetDetails ) {
-        if ( details.rules.modifyHeaders === 0 ) { continue; }
-        toFetch.push(fetchJSON(`/rulesets/modify-headers/${details.id}`));
-    }
-    const rulesets = await Promise.all(toFetch);
-
-    // Redirect rules can only be enforced with omnipotence
-    const allRules = [];
-    if ( hasOmnipotence ) {
-        for ( const rules of rulesets ) {
-            if ( Array.isArray(rules) === false ) { continue; }
-            for ( const rule of rules ) {
-                allRules.push(rule);
-            }
-        }
-    }
-    if ( allRules.length === 0 ) { return; }
-
-    const validRules = await pruneInvalidRegexRules('modify-headers', allRules);
-    if ( validRules.length === 0 ) { return; }
-
-    ubolLog(`Add ${validRules.length} DNR modify-headers rules`);
-    addRules.push(...validRules);
-}
-
-/******************************************************************************/
-
 async function updateDynamicRules() {
     const currentRules = await dnr.getDynamicRules();
     const addRules = [];
     const removeRuleIds = [];
 
-    // Remove potentially left-over strict-block rules from previous version
+    // Remove potentially left-over rules from previous version
     for ( const rule of currentRules ) {
-        if ( isStrictBlockRule(rule) === false ) { continue; }
+        if ( rule.id >= SPECIAL_RULES_REALM ) { continue; }
         removeRuleIds.push(rule.id);
+        rule.id = 0;
     }
 
-    await Promise.all([
-        updateRegexRules(currentRules, addRules, removeRuleIds),
-        updateRemoveparamRules(currentRules, addRules, removeRuleIds),
-        updateRedirectRules(currentRules, addRules, removeRuleIds),
-        updateModifyHeadersRules(currentRules, addRules, removeRuleIds),
-    ]);
+    await updateRegexRules(currentRules, addRules, removeRuleIds);
     if ( addRules.length === 0 && removeRuleIds.length === 0 ) { return; }
 
-    const maxRegexRuleCount = dnr.MAX_NUMBER_OF_REGEX_RULES;
-    let regexRuleCount = 0;
+    dynamicRegexCount = 0;
     let ruleId = 1;
     for ( const rule of addRules ) {
-        if ( rule?.condition.regexFilter ) { regexRuleCount += 1; }
-        if ( (rule.id || 0) >= TRUSTED_DIRECTIVE_BASE_RULE_ID ) { continue; }
+        if ( rule?.condition.regexFilter ) { dynamicRegexCount += 1; }
         rule.id = ruleId++;
     }
-    if ( regexRuleCount !== 0 ) {
-        ubolLog(`Using ${regexRuleCount}/${maxRegexRuleCount} dynamic regex-based DNR rules`);
+    if ( dynamicRegexCount !== 0 ) {
+        ubolLog(`Using ${dynamicRegexCount}/${dnr.MAX_NUMBER_OF_REGEX_RULES} dynamic regex-based DNR rules`);
     }
-    return Promise.all([
-        dnr.updateDynamicRules({ addRules, removeRuleIds }).then(( ) => {
-            if ( removeRuleIds.length !== 0 ) {
-                ubolLog(`Remove ${removeRuleIds.length} dynamic DNR rules`);
-            }
-            if ( addRules.length !== 0 ) {
-                ubolLog(`Add ${addRules.length} dynamic DNR rules`);
-            }
-        }).catch(reason => {
-            console.error(`updateDynamicRules() / ${reason}`);
-        }),
-        updateSessionRules(),
-    ]);
+
+    const response = {};
+
+    try {
+        await dnr.updateDynamicRules({ addRules, removeRuleIds });
+        if ( removeRuleIds.length !== 0 ) {
+            ubolLog(`Remove ${removeRuleIds.length} dynamic DNR rules`);
+        }
+        if ( addRules.length !== 0 ) {
+            ubolLog(`Add ${addRules.length} dynamic DNR rules`);
+        }
+    } catch(reason) {
+        ubolErr(`updateDynamicRules/${reason}`);
+        response.error = `${reason}`;
+    }
+
+    const result = await updateSessionRules();
+    if ( result?.error ) {
+        response.error ||= result.error;
+    }
+
+    return response;
+}
+
+/******************************************************************************/
+
+async function getEffectiveDynamicRules() {
+    const allRules = await dnr.getDynamicRules();
+    const dynamicRules = [];
+    for ( const rule of allRules ) {
+        if ( rule.id >= USER_RULES_BASE_RULE_ID ) { continue; }
+        dynamicRules.push(rule);
+    }
+    return dynamicRules;
 }
 
 /******************************************************************************/
@@ -342,13 +237,18 @@ async function updateStrictBlockRules(currentRules, addRules, removeRuleIds) {
 
     if ( rulesetConfig.strictBlockMode === false ) { return; }
 
+    // https://github.com/uBlockOrigin/uBOL-home/issues/428#issuecomment-3172663563
+    // https://bugs.webkit.org/show_bug.cgi?id=298199
+    // https://developer.apple.com/forums/thread/756214
+    if ( webextFlavor === 'safari' ) { return; }
+
     const [
         hasOmnipotence,
         rulesetDetails,
         permanentlyExcluded = [],
         temporarilyExcluded = [],
     ] = await Promise.all([
-        browser.permissions.contains({ origins: [ '<all_urls>' ] }),
+        hasBroadHostPermissions(),
         getEnabledRulesetsDetails(),
         localRead('excludedStrictBlockHostnames'),
         sessionRead('excludedStrictBlockHostnames'),
@@ -383,12 +283,13 @@ async function updateStrictBlockRules(currentRules, addRules, removeRuleIds) {
     if ( validRules.length === 0 ) { return; }
     ubolLog(`Add ${validRules.length} DNR strictblock rules`);
     for ( const rule of validRules ) {
+        rule.priority = STRICTBLOCK_PRIORITY;
         addRules.push(rule);
     }
 
     const allExcluded = permanentlyExcluded.concat(temporarilyExcluded);
     if ( allExcluded.length === 0 ) { return; }
-    addRules.push({
+    addRules.unshift({
         action: { type: 'allow' },
         condition: {
             requestDomains: allExcluded,
@@ -409,9 +310,11 @@ async function excludeFromStrictBlock(hostname, permanent) {
     return updateSessionRules();
 }
 
-async function setStrictBlockMode(state) {
+async function setStrictBlockMode(state, force = false) {
     const newState = Boolean(state);
-    if ( newState === rulesetConfig.strictBlockMode ) { return; }
+    if ( force === false ) {
+        if ( newState === rulesetConfig.strictBlockMode ) { return; }
+    }
     rulesetConfig.strictBlockMode = newState;
     const promises = [ saveRulesetConfig() ];
     if ( newState === false ) {
@@ -427,156 +330,86 @@ async function setStrictBlockMode(state) {
 /******************************************************************************/
 
 async function updateSessionRules() {
-    const addRules = [];
+    const addRulesUnfiltered = [];
     const removeRuleIds = [];
     const currentRules = await dnr.getSessionRules();
-    await updateStrictBlockRules(currentRules, addRules, removeRuleIds);
-    if ( addRules.length === 0 && removeRuleIds.length === 0 ) { return; }
-    const maxRegexRuleCount = dnr.MAX_NUMBER_OF_REGEX_RULES;
-    let regexRuleCount = 0;
+    await updateStrictBlockRules(currentRules, addRulesUnfiltered, removeRuleIds);
+    if ( addRulesUnfiltered.length === 0 && removeRuleIds.length === 0 ) { return; }
+    const maxRegexCount = dnr.MAX_NUMBER_OF_REGEX_RULES * 0.80;
+    let regexCount = dynamicRegexCount;
     let ruleId = 1;
-    for ( const rule of addRules ) {
-        if ( rule?.condition.regexFilter ) { regexRuleCount += 1; }
-        rule.id = ruleId++;
+    for ( const rule of addRulesUnfiltered ) {
+        if ( rule?.condition.regexFilter ) { regexCount += 1; }
+        rule.id = regexCount < maxRegexCount ? ruleId++ : 0;
     }
-    if ( regexRuleCount !== 0 ) {
-        ubolLog(`Using ${regexRuleCount}/${maxRegexRuleCount} session regex-based DNR rules`);
+    sessionRegexCount = regexCount - dynamicRegexCount;
+    const addRules = addRulesUnfiltered.filter(a => a.id !== 0);
+    const rejectedRuleCount = addRulesUnfiltered.length - addRules.length;
+    if ( rejectedRuleCount !== 0 ) {
+        ubolLog(`Too many regex-based filters, ${rejectedRuleCount} session rules dropped`);
     }
-    return dnr.updateSessionRules({ addRules, removeRuleIds }).then(( ) => {
+    if ( sessionRegexCount !== 0 ) {
+        ubolLog(`Using ${sessionRegexCount}/${dnr.MAX_NUMBER_OF_REGEX_RULES} session regex-based DNR rules`);
+    }
+    const response = {};
+    try {
+        await dnr.updateSessionRules({ addRules, removeRuleIds });
         if ( removeRuleIds.length !== 0 ) {
             ubolLog(`Remove ${removeRuleIds.length} session DNR rules`);
         }
         if ( addRules.length !== 0 ) {
             ubolLog(`Add ${addRules.length} session DNR rules`);
         }
-    }).catch(reason => {
-        console.error(`updateSessionRules() / ${reason}`);
-    });
+    } catch(reason) {
+        ubolErr(`updateSessionRules/${reason}`);
+        response.error = `${reason}`;
+    }
+    return response;
+}
+
+/******************************************************************************/
+
+async function getEffectiveSessionRules() {
+    const allRules = await dnr.getSessionRules();
+    const sessionRules = [];
+    for ( const rule of allRules ) {
+        if ( rule.id >= USER_RULES_BASE_RULE_ID ) { continue; }
+        sessionRules.push(rule);
+    }
+    return sessionRules;
 }
 
 /******************************************************************************/
 
 async function filteringModesToDNR(modes) {
-    const [
-        dynamicRules,
-        sessionRules,
-    ] = await Promise.all([
-        dnr.getDynamicRules({ ruleIds: [ TRUSTED_DIRECTIVE_BASE_RULE_ID+0 ] }),
-        dnr.getSessionRules({ ruleIds: [ TRUSTED_DIRECTIVE_BASE_RULE_ID+1 ] }),
-    ]);
-    const dynamicRule = dynamicRules?.length && dynamicRules[0] || undefined;
-    const beforeRequestDomainSet = new Set(dynamicRule?.condition.requestDomains);
-    const beforeExcludedRrequestDomainSet = new Set(dynamicRule?.condition.excludedRequestDomains);
-    if ( dynamicRule !== undefined && beforeRequestDomainSet.size === 0 ) {
-        beforeRequestDomainSet.add('all-urls');
-    } else {
-        beforeExcludedRrequestDomainSet.add('all-urls');
-    }
-
     const noneHostnames = new Set([ ...modes.none ]);
     const notNoneHostnames = new Set([ ...modes.basic, ...modes.optimal, ...modes.complete ]);
-    let afterRequestDomainSet = new Set();
-    let afterExcludedRequestDomainSet = new Set();
-    if ( noneHostnames.has('all-urls') ) {
-        afterRequestDomainSet = new Set([ 'all-urls' ]);
-        afterExcludedRequestDomainSet = notNoneHostnames;
+    const requestDomains = [];
+    const excludedRequestDomains = [];
+    const allowEverywhere = noneHostnames.has('all-urls');
+    if ( allowEverywhere ) {
+        excludedRequestDomains.push(...notNoneHostnames);
     } else {
-        afterRequestDomainSet = noneHostnames;
-        afterExcludedRequestDomainSet = new Set();
+        requestDomains.push(...noneHostnames);
     }
-
-    const removeDynamicRuleIds = [];
-    const removeSessionRuleIds = [];
-    if ( dynamicRule ) {
-        removeDynamicRuleIds.push(TRUSTED_DIRECTIVE_BASE_RULE_ID+0);
-        removeSessionRuleIds.push(TRUSTED_DIRECTIVE_BASE_RULE_ID+1);
-    }
-
-    const allowEverywhere = afterRequestDomainSet.delete('all-urls');
-    const addDynamicRules = [];
-    const addSessionRules = [];
-    if (
-        allowEverywhere ||
-        afterRequestDomainSet.size !== 0 ||
-        afterExcludedRequestDomainSet.size !== 0
-    ) {
-        const rule0 = {
-            id: TRUSTED_DIRECTIVE_BASE_RULE_ID+0,
-            action: { type: 'allowAllRequests' },
-            condition: {
-                resourceTypes: [ 'main_frame' ],
-            },
-            priority: 100,
-        };
-        if ( afterRequestDomainSet.size !== 0 ) {
-            rule0.condition.requestDomains =
-                Array.from(afterRequestDomainSet).sort();
-        } else if ( afterExcludedRequestDomainSet.size !== 0 ) {
-            rule0.condition.excludedRequestDomains =
-                Array.from(afterExcludedRequestDomainSet).sort();
-        }
-        addDynamicRules.push(rule0);
-        // https://github.com/uBlockOrigin/uBOL-home/issues/114
-        // https://github.com/uBlockOrigin/uBOL-home/issues/247
-        const rule1 = {
-            id: TRUSTED_DIRECTIVE_BASE_RULE_ID+1,
-            action: { type: 'allow' },
-            condition: {
-                tabIds: [ TAB_ID_NONE ],
-            },
-            priority: 100,
-        };
-        if ( rule0.condition.requestDomains ) {
-            rule1.condition.initiatorDomains =
-                rule0.condition.requestDomains.slice();
-        } else if ( rule0.condition.excludedRequestDomains ) {
-            rule1.condition.excludedInitiatorDomains =
-                rule0.condition.excludedRequestDomains.slice();
-        }
-        addSessionRules.push(rule1);
-    }
-
-    const noneCount = noneHostnames.has('all-urls')
-        ? -notNoneHostnames.size
+    const noneCount = allowEverywhere
+        ? notNoneHostnames.size
         : noneHostnames.size;
-
-    const promises = [];
-    if ( isDifferentAllowRules(addDynamicRules, dynamicRules) ) {
-        promises.push(dnr.updateDynamicRules({
-            addRules: addDynamicRules,
-            removeRuleIds: removeDynamicRuleIds,
-        }));
-        ubolLog(`Add "allowAllRequests" dynamic rule for ${noneCount} sites`);
-    }
-    if ( isDifferentAllowRules(addSessionRules, sessionRules) ) {
-        promises.push(dnr.updateSessionRules({
-            addRules: addSessionRules,
-            removeRuleIds: removeSessionRuleIds,
-        }));
-        ubolLog(`Add "allow" session rule for ${noneCount} sites`);
-    }
-    if ( promises.length === 0 ) { return; }
-    return Promise.all(promises);
+    return dnr.setAllowAllRules(
+        TRUSTED_DIRECTIVE_BASE_RULE_ID,
+        requestDomains.sort(),
+        excludedRequestDomains.sort(),
+        allowEverywhere,
+        TRUSTED_DIRECTIVE_PRIORITY
+    ).then(modified => {
+        if ( modified === false ) { return; }
+        ubolLog(`${allowEverywhere ? 'Enabled' : 'Disabled'} DNR filtering for ${noneCount} sites`);
+    });
 }
 
-const isDifferentAllowRules = (a, b) => {
-    const pp = [
-        'requestDomains',
-        'excludedRequestDomains',
-        'initiatorDomains',
-        'excludedInitiatorDomains',
-    ];
-    for ( const p of pp ) {
-        const ac = a?.length && a[0].condition[p] || [];
-        const bc = b?.length && b[0].condition[p] || [];
-        if ( ac.join() !== bc.join() ) { return true; }
-    }
-    return false;
-};
-
 /******************************************************************************/
 
-async function defaultRulesetsFromLanguage() {
+export async function getDefaultRulesetsFromEnv() {
     const dropCountry = lang => {
         const pos = lang.indexOf('-');
         if ( pos === -1 ) { return lang; }
@@ -594,21 +427,32 @@ async function defaultRulesetsFromLanguage() {
         `\\b(${Array.from(langSet).join('|')})\\b`
     );
 
-    const manifest = runtime.getManifest();
-    const rulesets = manifest.declarative_net_request.rule_resources;
+    const reMobile = /\bMobile\b/.test(navigator.userAgent)
+        ? /\bmobile\b/
+        : null
+
     const rulesetDetails = await getRulesetDetails();
     const out = [];
-    for ( const ruleset of rulesets ) {
+    for ( const ruleset of rulesetDetails.values() ) {
         const { id, enabled } = ruleset;
         if ( enabled ) {
             out.push(id);
             continue;
         }
-        const details = rulesetDetails.get(id);
-        if ( typeof details.lang !== 'string' ) { continue; }
-        if ( reTargetLang.test(details.lang) === false ) { continue; }
-        out.push(id);
+        if ( typeof ruleset.lang === 'string' ) {
+            if ( reTargetLang.test(ruleset.lang) ) {
+                out.push(id);
+                continue;
+            }
+        }
+        if ( typeof ruleset.tags === 'string' ) {
+            if ( reMobile?.test(ruleset.tags) ) {
+                out.push(id);
+                continue;
+            }
+        }
     }
+   
     return out;
 }
 
@@ -618,15 +462,12 @@ async function patchDefaultRulesets() {
     const [
         oldDefaultIds = [],
         newDefaultIds,
+        staticRulesetIds,
     ] = await Promise.all([
         localRead('defaultRulesetIds'),
-        defaultRulesetsFromLanguage(),
+        getDefaultRulesetsFromEnv(),
+        getStaticRulesets().then(r => r.map(a => a.id)),
     ]);
-
-    const manifest = runtime.getManifest();
-    const validIds = new Set(
-        manifest.declarative_net_request.rule_resources.map(r => r.id)
-    );
     const toAdd = [];
     const toRemove = [];
     for ( const id of newDefaultIds ) {
@@ -638,7 +479,7 @@ async function patchDefaultRulesets() {
         toRemove.push(id);
     }
     for ( const id of rulesetConfig.enabledRulesets ) {
-        if ( validIds.has(id) ) { continue; }
+        if ( staticRulesetIds.includes(id) ) { continue; }
         toRemove.push(id);
     }
     localWrite('defaultRulesetIds', newDefaultIds);
@@ -655,7 +496,11 @@ async function patchDefaultRulesets() {
 
 async function enableRulesets(ids) {
     const afterIds = new Set(ids);
-    const [ beforeIds, adminIds, rulesetDetails ] = await Promise.all([
+    const [
+        beforeIds,
+        adminIds,
+        rulesetDetails,
+    ] = await Promise.all([
         dnr.getEnabledRulesets().then(ids => new Set(ids)),
         getAdminRulesets(),
         getRulesetDetails(),
@@ -693,9 +538,7 @@ async function enableRulesets(ids) {
         disableRulesetSet.delete(id);
     }
 
-    if ( enableRulesetSet.size === 0 && disableRulesetSet.size === 0 ) {
-        return false;
-    }
+    if ( enableRulesetSet.size === 0 && disableRulesetSet.size === 0 ) { return; }
 
     const enableRulesetIds = Array.from(enableRulesetSet);
     const disableRulesetIds = Array.from(disableRulesetSet);
@@ -706,11 +549,41 @@ async function enableRulesets(ids) {
     if ( disableRulesetIds.length !== 0 ) {
         ubolLog(`Disable ruleset: ${disableRulesetIds}`);
     }
-    await dnr.updateEnabledRulesets({ enableRulesetIds, disableRulesetIds });
 
-    await updateDynamicRules();
+    const response = {};
 
-    return true;
+    await dnr.updateEnabledRulesets({
+        enableRulesetIds,
+        disableRulesetIds,
+    }).catch(reason => {
+        ubolErr(`updateEnabledRulesets/${reason}`);
+        response.error = `${reason}`;
+    });
+
+    const result = await updateDynamicRules();
+    if ( result?.error ) {
+        response.error ||= result.error;
+    }
+
+    await dnr.getEnabledRulesets().then(enabledRulesets => {
+        ubolLog(`Enabled rulesets: ${enabledRulesets}`);
+        response.enabledRulesets = enabledRulesets;
+        return dnr.getAvailableStaticRuleCount();
+    }).then(count => {
+        ubolLog(`Available static rule count: ${count}`);
+        response.staticRuleCount = count;
+    }).catch(reason => {
+        ubolErr(`getEnabledRulesets/${reason}`);
+    });
+
+    return response;
+}
+
+/******************************************************************************/
+
+async function getStaticRulesets() {
+    const manifest = runtime.getManifest();
+    return manifest.declarative_net_request.rule_resources;
 }
 
 /******************************************************************************/
@@ -734,15 +607,96 @@ async function getEnabledRulesetsDetails() {
 
 /******************************************************************************/
 
+async function getEffectiveUserRules() {
+    const allRules = await dnr.getDynamicRules();
+    const userRules = [];
+    for ( const rule of allRules ) {
+        if ( rule.id < USER_RULES_BASE_RULE_ID ) { continue; }
+        userRules.push(rule);
+    }
+    return userRules;
+}
+
+async function updateUserRules() {
+    const [
+        userRules,
+        userRulesText = '',
+    ] = await Promise.all([
+        getEffectiveUserRules(),
+        localRead('userDnrRules'),
+    ]);
+
+    const effectiveRulesText = rulesetConfig.developerMode
+        ? userRulesText
+        : '';
+
+    const parsed = rulesFromText(effectiveRulesText);
+    const { rules } = parsed;
+    const removeRuleIds = [ ...userRules.map(a => a.id) ];
+    const rejectedRegexes = [];
+    const addRules = await pruneInvalidRegexRules('user', rules, rejectedRegexes);
+    const out = { added: 0, removed: 0, errors: [] };
+
+    if ( rejectedRegexes.length !== 0 ) {
+        rejectedRegexes.forEach(e =>
+            out.errors.push(`regexFilter: ${e.regex} → ${e.reason}`)
+        );
+    }
+
+    if ( removeRuleIds.length === 0 && addRules.length === 0 ) {
+        await localRemove('userDnrRuleCount');
+        return out;
+    }
+
+    let ruleId = 0;
+    for ( const rule of addRules ) {
+        rule.id = USER_RULES_BASE_RULE_ID + ruleId++;
+        rule.priority = (rule.priority || 1) + USER_RULES_PRIORITY;
+    }
+
+    // Rules are first removed separately to ensure registered rules match
+    // user rules text. A bad rule in user rules text would prevent the
+    // rules from being removed if the removal was done at the same time as
+    // adding rules.
+    try {
+        await dnr.updateDynamicRules({ removeRuleIds });
+        await dnr.updateDynamicRules({ addRules });
+        if ( removeRuleIds.length !== 0 ) {
+            ubolLog(`updateUserRules() / Removed ${removeRuleIds.length} dynamic DNR rules`);
+        }
+        if ( addRules.length !== 0 ) {
+            ubolLog(`updateUserRules() / Added ${addRules.length} DNR rules`);
+        }
+        out.added = addRules.length;
+        out.removed = removeRuleIds.length;
+    } catch(reason) {
+        ubolErr(`updateUserRules/${reason}`);
+        out.errors.push(`${reason}`);
+    } finally {
+        const userRules = await getEffectiveUserRules();
+        if ( userRules.length === 0 ) {
+            await localRemove('userDnrRuleCount');
+        } else {
+            await localWrite('userDnrRuleCount', addRules.length);
+        }
+    }
+    return out;
+}
+
+/******************************************************************************/
+
 export {
-    defaultRulesetsFromLanguage,
     enableRulesets,
     excludeFromStrictBlock,
     filteringModesToDNR,
-    getRulesetDetails,
+    getEffectiveDynamicRules,
+    getEffectiveSessionRules,
+    getEffectiveUserRules,
     getEnabledRulesetsDetails,
+    getRulesetDetails,
     patchDefaultRulesets,
     setStrictBlockMode,
     updateDynamicRules,
     updateSessionRules,
+    updateUserRules,
 };
