@@ -26,10 +26,15 @@
 (function uBOL_cssGeneric() {
 
 const genericSelectorMap = self.genericSelectorMap || new Map();
-delete self.genericSelectorMap;
-
+self.genericSelectorMap = undefined;
 if ( genericSelectorMap.size === 0 ) { return; }
 
+const genericExceptionSieve = self.genericExceptionSieve || new Set();
+self.genericExceptionSieve = undefined;
+
+const genericExceptionMap = self.genericExceptionMap || new Map();
+self.genericExceptionMap = undefined;
+
 /******************************************************************************/
 
 const maxSurveyTimeSlice = 4;
@@ -56,7 +61,7 @@ const hashFromStr = (type, s) => {
     for ( let i = 0; i < len; i += step ) {
         hash = (hash << 5) + hash ^ s.charCodeAt(i);
     }
-    return hash & 0xFFFFFF;
+    return hash & 0xFFF;
 };
 
 /******************************************************************************/
@@ -76,7 +81,11 @@ const uBOL_idFromNode = (node, out) => {
     const selectorList = genericSelectorMap.get(hash);
     if ( selectorList === undefined ) { return; }
     genericSelectorMap.delete(hash);
-    out.push(selectorList);
+    if ( genericExceptionSieve.has(hash) ) {
+        applyExceptions(selectorList, out);
+    } else {
+        out.push(selectorList);
+    }
 };
 
 // https://github.com/uBlockOrigin/uBlock-issues/discussions/2076
@@ -96,10 +105,28 @@ const uBOL_classesFromNode = (node, out) => {
         const selectorList = genericSelectorMap.get(hash);
         if ( selectorList === undefined ) { continue; }
         genericSelectorMap.delete(hash);
-        out.push(selectorList);
+        if ( genericExceptionSieve.has(hash) ) {
+            applyExceptions(selectorList, out);
+        } else {
+            out.push(selectorList);
+        }
     }
 };
 
+const applyExceptions = (selectorList, out) => {
+    const selectors = new Set(selectorList.split(',\n'));
+    self.isolatedAPI.forEachHostname(hostname => {
+        const exceptions = genericExceptionMap.get(hostname);
+        if ( exceptions === undefined ) { return; }
+        for ( const exception of exceptions.split('\n') ) {
+            selectors.delete(exception);
+        }
+        if ( selectors.size === 0 ) { return true; }
+    }, { hasEntities: true });
+    if ( selectors.size === 0 ) { return; }
+    out.push(Array.from(selectors).join(',\n'));
+}
+
 /******************************************************************************/
 
 const pendingNodes = {
@@ -161,7 +188,7 @@ const uBOL_processNodes = ( ) => {
     if ( styleSheetTimer !== undefined ) { return; }
     styleSheetTimer = self.requestAnimationFrame(( ) => {
         styleSheetTimer = undefined;
-        uBOL_injectCSS(`${styleSheetSelectors.join(',')}{display:none!important;}`);
+        self.cssAPI.insert(`${styleSheetSelectors.join(',')}{display:none!important;}`);
         styleSheetSelectors.length = 0;
     });
 };
@@ -187,12 +214,15 @@ const uBOL_processChanges = mutations => {
 
 /******************************************************************************/
 
-const uBOL_injectCSS = (css, count = 10) => {
-    chrome.runtime.sendMessage({ what: 'insertCSS', css }).catch(( ) => {
-        count -= 1;
-        if ( count === 0 ) { return; }
-        uBOL_injectCSS(css, count - 1);
-    });
+const stopAll = ( ) => {
+    if ( domChangeTimer !== undefined ) {
+        self.clearTimeout(domChangeTimer);
+        domChangeTimer = undefined;
+    }
+    domMutationObserver.disconnect();
+    domMutationObserver.takeRecords();
+    domMutationObserver = undefined;
+    genericSelectorMap.clear();
 };
 
 /******************************************************************************/
@@ -219,20 +249,6 @@ needDomChangeObserver();
 
 /******************************************************************************/
 
-const stopAll = reason => {
-    if ( domChangeTimer !== undefined ) {
-        self.clearTimeout(domChangeTimer);
-        domChangeTimer = undefined;
-    }
-    domMutationObserver.disconnect();
-    domMutationObserver.takeRecords();
-    domMutationObserver = undefined;
-    genericSelectorMap.clear();
-    console.info(`uBOL: Generic cosmetic filtering stopped because ${reason}`);
-};
-
-/******************************************************************************/
-
 })();
 
 /******************************************************************************/
