@@ -19,39 +19,27 @@
     Home: https://github.com/gorhill/uBlock
 */
 
-/* globals browser */
-
-'use strict';
-
 /******************************************************************************/
 
-import µb from './background.js';
+import {
+    domainFromHostname,
+    hostnameFromURI,
+} from './uri-utils.js';
+
+import { MRUCache } from './mrucache.js';
+import { ScriptletFilteringEngine } from './scriptlet-filtering-core.js';
+
 import logger from './logger.js';
 import { onBroadcast } from './broadcast.js';
 import { redirectEngine as reng } from './redirect-engine.js';
 import { sessionFirewall } from './filtering-engines.js';
-import { MRUCache } from './mrucache.js';
-import { ScriptletFilteringEngine } from './scriptlet-filtering-core.js';
-
-import {
-    domainFromHostname,
-    entityFromDomain,
-    hostnameFromURI,
-} from './uri-utils.js';
+import µb from './background.js';
 
 /******************************************************************************/
 
-const contentScriptRegisterer = new (class {
-    constructor() {
-        this.hostnameToDetails = new Map();
-        if ( browser.contentScripts === undefined ) { return; }
-        onBroadcast(msg => {
-            if ( msg.what !== 'filteringBehaviorChanged' ) { return; }
-            if ( msg.direction > 0 ) { return; }
-            if ( msg.hostname ) { return this.flush(msg.hostname); }
-            this.reset();
-        });
-    }
+const contentScriptRegisterer = {
+    id: 1,
+    hostnameToDetails: new Map(),
     register(hostname, code) {
         if ( browser.contentScripts === undefined ) { return false; }
         if ( hostname === '' ) { return false; }
@@ -60,9 +48,10 @@ const contentScriptRegisterer = new (class {
             if ( code === details.code ) {
                 return details.handle instanceof Promise === false;
             }
-            details.handle.unregister();
+            this.unregisterHandle(details.handle);
             this.hostnameToDetails.delete(hostname);
         }
+        const id = this.id++;
         const promise = browser.contentScripts.register({
             js: [ { code } ],
             allFrames: true,
@@ -70,21 +59,26 @@ const contentScriptRegisterer = new (class {
             matchAboutBlank: true,
             runAt: 'document_start',
         }).then(handle => {
-            this.hostnameToDetails.set(hostname, { handle, code });
+            const details = this.hostnameToDetails.get(hostname);
+            if ( details === undefined ) { return; }
+            if ( details.id !== id ) { return; }
+            details.handle = handle;
         }).catch(( ) => {
             this.hostnameToDetails.delete(hostname);
         });
-        this.hostnameToDetails.set(hostname, { handle: promise, code });
+        this.hostnameToDetails.set(hostname, { id, handle: promise, code });
         return false;
-    }
+    },
     unregister(hostname) {
+        if ( hostname === '' ) { return; }
         if ( this.hostnameToDetails.size === 0 ) { return; }
         const details = this.hostnameToDetails.get(hostname);
         if ( details === undefined ) { return; }
         this.hostnameToDetails.delete(hostname);
         this.unregisterHandle(details.handle);
-    }
+    },
     flush(hostname) {
+        if ( hostname === '' ) { return; }
         if ( hostname === '*' ) { return this.reset(); }
         for ( const hn of this.hostnameToDetails.keys() ) {
             if ( hn.endsWith(hostname) === false ) { continue; }
@@ -92,56 +86,27 @@ const contentScriptRegisterer = new (class {
             if ( pos !== 0 && hn.charCodeAt(pos-1) !== 0x2E /* . */ ) { continue; }
             this.unregister(hn);
         }
-    }
+    },
     reset() {
         if ( this.hostnameToDetails.size === 0 ) { return; }
         for ( const details of this.hostnameToDetails.values() ) {
             this.unregisterHandle(details.handle);
         }
         this.hostnameToDetails.clear();
-    }
+    },
     unregisterHandle(handle) {
         if ( handle instanceof Promise ) {
-            handle.then(handle => { handle.unregister(); });
+            handle.then(handle => {
+                if ( handle ) { handle.unregister(); }
+            });
         } else {
             handle.unregister();
         }
-    }
-})();
+    },
+};
 
 /******************************************************************************/
 
-const mainWorldInjector = (( ) => {
-    const parts = [
-        '(',
-        function(injector, details) {
-            if ( typeof self.uBO_scriptletsInjected === 'string' ) { return; }
-            const doc = document;
-            if ( doc.location === null ) { return; }
-            const hostname = doc.location.hostname;
-            if ( hostname !== '' && details.hostname !== hostname ) { return; }
-            injector(doc, details);
-            return 0;
-        }.toString(),
-        ')(',
-            vAPI.scriptletsInjector, ', ',
-            'json-slot',
-        ');',
-    ];
-    return {
-        parts,
-        jsonSlot: parts.indexOf('json-slot'),
-        assemble: function(hostname, scriptlets, filters) {
-            this.parts[this.jsonSlot] = JSON.stringify({
-                hostname,
-                scriptlets,
-                filters,
-            });
-            return this.parts.join('');
-        },
-    };
-})();
-
 const isolatedWorldInjector = (( ) => {
     const parts = [
         '(',
@@ -160,24 +125,64 @@ const isolatedWorldInjector = (( ) => {
             'json-slot',
         ');',
     ];
+    const jsonSlot = parts.indexOf('json-slot');
     return {
-        parts,
-        jsonSlot: parts.indexOf('json-slot'),
-        assemble: function(hostname, scriptlets) {
-            this.parts[this.jsonSlot] = JSON.stringify({ hostname });
-            const code = this.parts.join('');
+        assemble(hostname, details) {
+            parts[jsonSlot] = JSON.stringify({ hostname });
+            const code = parts.join('');
             // Manually substitute noop function with scriptlet wrapper
             // function, so as to not suffer instances of special
             // replacement characters `$`,`\` when using String.replace()
             // with scriptlet code.
             const match = /function\(\)\{\}/.exec(code);
             return code.slice(0, match.index) +
-                scriptlets +
+                details.isolatedWorld +
                 code.slice(match.index + match[0].length);
         },
     };
 })();
 
+const onScriptletMessageInjector = (( ) => {
+    const parts = [
+        '(',
+        function(name) {
+            if ( self.uBO_bcSecret ) { return; }
+            try {
+                const bcSecret = new self.BroadcastChannel(name);
+                bcSecret.onmessage = ev => {
+                    const msg = ev.data;
+                    switch ( typeof msg ) {
+                    case 'string':
+                        if ( msg !== 'areyouready?' ) { break; }
+                        bcSecret.postMessage('iamready!');
+                        break;
+                    case 'object':
+                        if ( self.vAPI && self.vAPI.messaging ) {
+                            self.vAPI.messaging.send('contentscript', msg);
+                        } else {
+                            console.log(`[uBO][${msg.type}]${msg.text}`);
+                        }
+                        break;
+                    }
+                };
+                bcSecret.postMessage('iamready!');
+                self.uBO_bcSecret = bcSecret;
+            } catch {
+            }
+        }.toString(),
+        ')(',
+            'bcSecret-slot',
+        ');',
+    ];
+    const bcSecretSlot = parts.indexOf('bcSecret-slot');
+    return {
+        assemble(details) {
+            parts[bcSecretSlot] = JSON.stringify(details.bcSecret);
+            return parts.join('\n');
+        },
+    };
+})();
+
 /******************************************************************************/
 
 export class ScriptletFilteringEngineEx extends ScriptletFilteringEngine {
@@ -187,23 +192,60 @@ export class ScriptletFilteringEngineEx extends ScriptletFilteringEngine {
         this.warSecret = undefined;
         this.scriptletCache = new MRUCache(32);
         this.isDevBuild = undefined;
-        onBroadcast(msg => {
-            if ( msg.what !== 'hiddenSettingsChanged' ) { return; }
-            this.scriptletCache.reset();
-            this.isDevBuild = undefined;
+        this.logLevel = 1;
+        this.bc = onBroadcast(msg => {
+            switch ( msg.what ) {
+            case 'filteringBehaviorChanged': {
+                const direction = msg.direction || 0;
+                if ( direction > 0 ) { return; }
+                if ( direction >= 0 && msg.hostname ) {
+                    return contentScriptRegisterer.flush(msg.hostname);
+                }
+                contentScriptRegisterer.reset();
+                break;
+            }
+            case 'hiddenSettingsChanged':
+                this.isDevBuild = undefined;
+                /* fall through */
+            case 'loggerEnabled':
+            case 'loggerDisabled':
+                this.clearCache();
+                break;
+            case 'loggerLevelChanged':
+                this.logLevel = msg.level;
+                vAPI.tabs.query({
+                    discarded: false,
+                    url: [ 'http://*/*', 'https://*/*' ],
+                }).then(tabs => {
+                    for ( const tab of tabs ) {
+                        const { status } = tab;
+                        if ( status !== 'loading' && status !== 'complete' ) { continue; }
+                        vAPI.tabs.executeScript(tab.id, {
+                            allFrames: true,
+                            file: `/js/scriptlets/scriptlet-loglevel-${this.logLevel}.js`,
+                            matchAboutBlank: true,
+                        });
+                    }
+                });
+                this.clearCache();
+                break;
+            }
         });
     }
 
     reset() {
         super.reset();
         this.warSecret = vAPI.warSecret.long(this.warSecret);
-        this.scriptletCache.reset();
-        contentScriptRegisterer.reset();
+        this.clearCache();
     }
 
     freeze() {
         super.freeze();
         this.warSecret = vAPI.warSecret.long(this.warSecret);
+        this.clearCache();
+    }
+
+    clearCache() {
         this.scriptletCache.reset();
         contentScriptRegisterer.reset();
     }
@@ -220,8 +262,7 @@ export class ScriptletFilteringEngineEx extends ScriptletFilteringEngine {
         }
 
         if ( this.scriptletCache.resetTime < reng.modifyTime ) {
-            this.warSecret = vAPI.warSecret.long(this.warSecret);
-            this.scriptletCache.reset();
+            this.clearCache();
         }
 
         let scriptletDetails = this.scriptletCache.lookup(hostname);
@@ -238,58 +279,85 @@ export class ScriptletFilteringEngineEx extends ScriptletFilteringEngine {
             this.warSecret = vAPI.warSecret.long();
         }
 
+        const bcSecret = vAPI.generateSecret(3);
+
         const options = {
-            scriptletGlobals: [
-                [ 'warOrigin', this.warOrigin ],
-                [ 'warSecret', this.warSecret ],
-            ],
+            scriptletGlobals: {
+                warOrigin: this.warOrigin,
+                warSecret: this.warSecret,
+            },
             debug: this.isDevBuild,
             debugScriptlets: µb.hiddenSettings.debugScriptlets,
         };
+        if ( logger.enabled ) {
+            options.scriptletGlobals.bcSecret = bcSecret;
+            options.scriptletGlobals.logLevel = this.logLevel;
+        }
 
         scriptletDetails = super.retrieve(request, options);
 
-        this.scriptletCache.add(hostname, scriptletDetails || null);
+        if ( scriptletDetails === undefined ) {
+            if ( request.nocache !== true ) {
+                this.scriptletCache.add(hostname, null);
+            }
+            return;
+        }
 
-        return scriptletDetails;
+        const contentScript = [];
+        if ( scriptletDetails.mainWorld ) {
+            contentScript.push(vAPI.scriptletsInjector(hostname, scriptletDetails));
+        }
+        if ( scriptletDetails.isolatedWorld ) {
+            contentScript.push(isolatedWorldInjector.assemble(hostname, scriptletDetails));
+        }
+
+        const cachedScriptletDetails = {
+            bcSecret,
+            code: contentScript.join('\n\n'),
+            filters: scriptletDetails.filters,
+        };
+
+        if ( hostname !== '' && request.nocache !== true ) {
+            this.scriptletCache.add(hostname, cachedScriptletDetails);
+        }
+
+        return cachedScriptletDetails;
     }
 
     injectNow(details) {
         if ( typeof details.frameId !== 'number' ) { return; }
 
-        const request = {
+        const hostname = hostnameFromURI(details.url);
+        const domain = domainFromHostname(hostname);
+
+        const scriptletDetails = this.retrieve({
             tabId: details.tabId,
             frameId: details.frameId,
             url: details.url,
-            hostname: hostnameFromURI(details.url),
-            domain: undefined,
-            entity: undefined
-        };
-
-        request.domain = domainFromHostname(request.hostname);
-        request.entity = entityFromDomain(request.domain);
-
-        const scriptletDetails = this.retrieve(request);
+            hostname,
+            domain,
+            ancestors: details.ancestors,
+        });
         if ( scriptletDetails === undefined ) {
-            contentScriptRegisterer.unregister(request.hostname);
+            contentScriptRegisterer.unregister(hostname);
             return;
         }
+        if ( Boolean(scriptletDetails.code) === false ) {
+            return scriptletDetails;
+        }
 
-        const contentScript = [];
+        const contentScript = [ scriptletDetails.code ];
+        if ( logger.enabled ) {
+            contentScript.unshift(
+                onScriptletMessageInjector.assemble(scriptletDetails)
+            );
+        }
         if ( µb.hiddenSettings.debugScriptletInjector ) {
-            contentScript.push('debugger');
+            contentScript.unshift('debugger');
         }
-        const { mainWorld = '', isolatedWorld = '', filters } = scriptletDetails;
-        if ( mainWorld !== '' ) {
-            contentScript.push(mainWorldInjector.assemble(request.hostname, mainWorld, filters));
-        }
-        if ( isolatedWorld !== '' ) {
-            contentScript.push(isolatedWorldInjector.assemble(request.hostname, isolatedWorld));
-        }
-
         const code = contentScript.join('\n\n');
 
-        const isAlreadyInjected = contentScriptRegisterer.register(request.hostname, code);
+        const isAlreadyInjected = contentScriptRegisterer.register(hostname, code);
         if ( isAlreadyInjected !== true ) {
             vAPI.tabs.executeScript(details.tabId, {
                 code,
@@ -298,24 +366,22 @@ export class ScriptletFilteringEngineEx extends ScriptletFilteringEngine {
                 runAt: 'document_start',
             });
         }
-
         return scriptletDetails;
     }
 
     toLogger(request, details) {
         if ( details === undefined ) { return; }
         if ( logger.enabled !== true ) { return; }
-        if ( typeof details.filters !== 'string' ) { return; }
-        const fctxt = µb.filteringContext
+        if ( Array.isArray(details.filters) === false ) { return; }
+        µb.filteringContext
             .duplicate()
             .fromTabId(request.tabId)
             .setRealm('extended')
             .setType('scriptlet')
             .setURL(request.url)
-            .setDocOriginFromURL(request.url);
-        for ( const raw of details.filters.split('\n') ) {
-            fctxt.setFilter({ source: 'extended', raw }).toLogger();
-        }
+            .setDocOriginFromURL(request.url)
+            .setFilter(details.filters.map(a => ({ source: 'extended', raw: a })))
+            .toLogger();
     }
 }
 
