@@ -19,10 +19,6 @@
     Home: https://github.com/gorhill/uBlock
 */
 
-/* globals browser */
-
-'use strict';
-
 /******************************************************************************/
 /******************************************************************************/
 
@@ -31,7 +27,7 @@
 /******************************************************************************/
 
 if ( typeof vAPI !== 'object' ) { return; }
-if ( typeof vAPI === null ) { return; }
+if ( vAPI === null ) { return; }
 if ( vAPI.domFilterer instanceof Object === false ) { return; }
 
 if ( vAPI.inspectorFrame ) { return; }
@@ -435,7 +431,7 @@ const elementsFromSelector = function(selector, context) {
     // plain CSS selector
     try {
         return context.querySelectorAll(selector);
-    } catch (ex) {
+    } catch {
     }
     return [];
 };
@@ -447,7 +443,7 @@ const elementsFromSpecialSelector = function(selector) {
         let nodes;
         try {
             nodes = document.querySelectorAll(matches[1]);
-        } catch(ex) {
+        } catch {
             nodes = [];
         }
         for ( const node of nodes ) {
@@ -776,7 +772,7 @@ const contentInspectorChannel = (( ) => {
         if ( toLoggerPort === undefined ) { return; }
         try {
             toLoggerPort.postMessage(msg);
-        } catch(_) {
+        } catch {
             shutdownInspector();
         }
     };
