@@ -21,6 +21,10 @@
 
 // https://www.reddit.com/r/uBlockOrigin/comments/ghjqph/
 // https://github.com/NanoMeow/QuickReports/issues/3717
+// https://www.reddit.com/r/uBlockOrigin/comments/qyx7en/
+
+// https://searchfox.org/mozilla-central/source/browser/extensions/webcompat/shims/apstag.js
+//   Import queue-related initialization code.
 
 (function() {
     'use strict';
@@ -28,9 +32,11 @@
     const noopfn = function() {
         ; // jshint ignore:line
     }.bind();
+    const _Q = w.apstag && w.apstag._Q || [];
     const apstag = {
+        _Q,
         fetchBids: function(a, b) {
-            if ( b instanceof Function ) {
+            if ( typeof b === 'function' ) {
                 b([]);
             }
         },
@@ -39,4 +45,18 @@
         targetingKeys: noopfn,
     };
     w.apstag = apstag;
+    _Q.push = function(prefix, args) {
+        try {
+            switch (prefix) {
+            case 'f':
+                apstag.fetchBids(...args);
+                break;
+            }
+        } catch (e) {
+            console.trace(e);
+        }
+    };
+    for ( const cmd of _Q ) {
+        _Q.push(cmd);
+    }
 })();
