@@ -68,12 +68,13 @@
         disableInitialLoad: noopfn,
         display: noopfn,
         enableAsyncRendering: noopfn,
+        enableLazyLoad: noopfn,
         enableSingleRequest: noopfn,
         enableSyncRendering: noopfn,
         enableVideoAds: noopfn,
         get: noopnullfn,
         getAttributeKeys: nooparrayfn,
-        getTargeting: noopfn,
+        getTargeting: nooparrayfn,
         getTargetingKeys: nooparrayfn,
         getSlots: nooparrayfn,
         refresh: noopfn,
